#***********************************************************************
#           Kreis Uhr (Tcl/Tk-Version)
#           ==========================
#               Uwe Berger; 2009
#
# Das Script setzt Tcl/Tk in der Version 8.5.x voraus!
# (wg. neuer Option -format bei clock scan...)
# 
# Anzeige von Uhrzeit sowie Sonnenaufgang-/untergang/-stand
#
# Die Kreise haben folgende Bedeutung (von aussen nach innen):
#	-Sekunden
#	-Minuten
#	-Stunden (ein Umlauf ueber 24h; 0 Uhr bei ist bei 6 Uhr
#	 der gewohnten 12h-Uhranzeige
#	-markiert Sonnenaufgang/-untergang, normalisiert auf den
#	 Stundenkreis (dazu sind die geografischen Koordinaten
#    des Standpunktes des Betrachters entsprechend zu setzen)
#
# Die jeweilige Skaleneinteilung entspicht der jeweiligen
# Einheit; die dabei gewohnten Zusammenfassungen sind mit einem
# dickeren Skalenstrich ausgefuehrt.
#
# Die Fortschreibung der Kreissegmente erfolgt im Uhrzeigersinn,
# was man aber, zur Verwirrung des Betrachters, noch aendern
# koennte...:-)))))
# 
# Die Anregung zu dieser Uhr bekam ich durch folgende Webseite:
#		http://wiki.tcl.tk/13226
#
# Die Formeln zur Berechnung des Standes der Sonne wurden hier
# gefunden:
#		http://lexikon.astronomie.info/zeitgleichung/
# (Achtung: bei den verwendeten Formeln handelt es sich um 
# Naehrungsgleichungen, genauere Berechnungsalgorithmen sind
# auf der gleichen Seite zu finden!)
#
# Die Formeln zur Bestimmung, ob momentan Sommer- oder Winterzeit
# ist, wurden hier gefunden:
#		http://wiki.tcl.tk/14617
# 
# ---------
# Have fun!
#
#***********************************************************************

package require Tk 8.5

# Definition GUI-Aussehen
set gvar(dx) 500
set gvar(dy) $gvar(dx)
set gvar(border) 30
set gvar(c_bg) black
set gvar(c_bg_border) grey20
set gvar(c_grid) $gvar(c_bg_border)
set gvar(c_second) green
set gvar(c_minute) red
set gvar(c_hour) blue
set gvar(c_sun) yellow

# Koordinaten Betrachtungsort (fuer Sonnenstandsberechnungen)
# (er sollte sich in der eingestellten Zeitzone des Rechners
# befinden)
set gvar(laenge) 12.509
set gvar(breite) 52.427

# interne Variablen
set gvar(old_m) 60
set gvar(old_doy) 400

#**********************************************************************
# Ermittlung, ob gerade Sommer- oder Winterzeit ist
# entnommen aus: http://wiki.tcl.tk/14617
# (keine Ahnung, ob das so wirklich funktioniert, nochmal testen...)
#**********************************************************************
proc is_dst {time tz} {
#	set year [clock format $time -format %Y -timezone $tz]
#	set midsummer [clock scan "21 06 $year" -format "%d %m %Y" -timezone $tz]
#	set midwinter [clock scan "21 12 $year" -format "%d %m %Y" -timezone $tz]
#	set midsummer_offset [scan [clock format $midsummer -format %z -timezone $tz] %d]
#	set midwinter_offset [scan [clock format $midwinter -format %z -timezone $tz] %d]
	set year [clock format $time -format %Y ]
	set midsummer [clock scan "21 06 $year" -format "%d %m %Y"]
	set midwinter [clock scan "21 12 $year" -format "%d %m %Y"]
	set midsummer_offset [scan [clock format $midsummer -format %z] %d]
	set midwinter_offset [scan [clock format $midwinter -format %z] %d]
#
	if {$midsummer_offset == $midwinter_offset} {
		return -code error "DST not in use in locale $tz"
	}
	if {$midsummer_offset < $midwinter_offset} {
		#-- In southern hemisphere
		set temp $midwinter_offset
		set midwinter_offset $midsummer_offset
		set midsummer_offset $temp
	}
#	set offset [scan [clock format $time -format %z -timezone $tz] %d]
	set offset [scan [clock format $time -format %z] %d]
#
	if {$offset == $midsummer_offset} {
		return 1
	} elseif {$offset == $midwinter_offset} {
		return 0
	} else {
		return -code error "Something crazy happened"
	}
}

#**********************************************************************
# Berechnungsroutinen Sonnenstand/-aufgang/-untergang
# entnommen aus: http://lexikon.astronomie.info/zeitgleichung/source.html
#**********************************************************************
# weitere (interne) Variablen...
set gvar(pi) 3.1415926536
set gvar(rad) [expr $gvar(pi) / 180.0]
set gvar(h) [expr -(50.0/60.0)*$gvar(rad)]
set gvar(B) [expr $gvar(breite)*$gvar(rad)]

proc sonnendeklination {T} {
	return [expr 0.409526325277017 * sin (0.0169060504029192 * ($T - 80.0856919827619))]
}

proc zeitdifferenz {Deklination} {
	global gvar
	return [expr 12.0*acos((sin($gvar(h))-sin($gvar(B))*sin($Deklination))/(cos($gvar(B))*cos($Deklination)))/$gvar(pi)]
}

proc zeitgleichung {T} {
	return [expr -0.170869921174742*sin(0.0336997028793971 * $T + 0.465419984181394) - 0.129890681040717*sin(0.0178674832556871*$T - 0.167936777524864)]
}

proc _aufgang {T} {
	return [expr 12 - [zeitdifferenz [sonnendeklination $T]] - [zeitgleichung $T]]
}

proc _untergang {T} {
	return [expr 12 + [zeitdifferenz [sonnendeklination $T]] - [zeitgleichung $T]]
}

proc aufgang {T} {
	global gvar
	return [expr [_aufgang $T] - $gvar(laenge) / 15.0 + $gvar(zone)]
}

proc untergang {T} {
	global gvar
	return [expr [_untergang $T] - $gvar(laenge) / 15.0 + $gvar(zone)]
}
#************* Ende Sonnenstand ************************************

# ******************************************************************
proc grid {scale scale_mark r1 r2} {
	global gvar
	set angle [expr 2*$gvar(pi)/$scale]
	for {set i 0} {$i < $scale} {incr i} {
		if {[expr $i % $scale_mark]} {set width 1} else {set width 3}
		.c create line	[expr cos($i*$angle)*$r1+$gvar(dx)/2]\
						[expr sin($i*$angle)*$r1+$gvar(dx)/2]\
						[expr cos($i*$angle)*$r2+$gvar(dx)/2]\
						[expr sin($i*$angle)*$r2+$gvar(dx)/2]\
						-fill $gvar(c_grid)\
						-width $width
	}				
}

# ******************************************************************
proc update_clock w {
	global gvar
	scan [clock format [clock sec] -format %H] %d h
	scan [clock format [clock sec] -format %M] %d m
	scan [clock format [clock sec] -format %S] %d s
	scan [clock format [clock sec] -format %j] %d day_of_year
	# Sekundenkreis
	$w itemconfig second -extent [expr $s*-6]
	# Minutenkreis
	if {$gvar(old_m) != $m} {
		set gvar(old_m) $m
		$w itemconfig minute -extent [expr $m*-6]
		# auch Stundenkreis anpassen
		$w itemconfig hour -extent [expr ($h+$m/60.)*-15]
	}
	# Sonnenaufgang/-untergang
	if {$gvar(old_doy) != $day_of_year} {
		set gvar(old_doy) $day_of_year
		if {[is_dst [clock scan now] ""]} {set gvar(zone) 2} else {set gvar(zone) 1}
		$w itemconfig sun \
					-start [expr 270+[aufgang $day_of_year]*-15]\
					-extent [expr ([untergang $day_of_year]-[aufgang $day_of_year])*-15] 
	}
}

# ******************************************************************
proc every {ms body} {eval $body; after $ms [info level 0]}

# ******************************************************************
# GUI
# ******************************************************************
	wm title . "Kreis Uhr" 
	wm resizable . 0 0
	# Zeichenflaeche
	canvas .c -width $gvar(dx) -height $gvar(dy) -bg $gvar(c_bg) -bd 0
	pack .c
	# Sekundenkreis
	set i 1
	.c create oval	[expr $gvar(border) * $i]\
					[expr $gvar(border) * $i]\
					[expr $gvar(dx) - ($i * $gvar(border))]\
					[expr $gvar(dy) - ($i * $gvar(border))]\
					-fill $gvar(c_bg) -outline $gvar(c_bg_border)
	.c create arc	[expr $gvar(border) * $i]\
					[expr $gvar(border) * $i]\
					[expr $gvar(dx) - ($i * $gvar(border))]\
					[expr $gvar(dy) - ($i * $gvar(border))]\
					-fill $gvar(c_second) -outline $gvar(c_second)\
					-start 90 -tag second
	grid 60 5 [expr $gvar(dx)/2 - ($i * $gvar(border))] [expr $gvar(dx)/2 - (($i+1) * $gvar(border))]
	# Minutenkreis
	incr i
	.c create oval	[expr $gvar(border) * $i]\
					[expr $gvar(border) * $i]\
					[expr $gvar(dx) - ($i * $gvar(border))]\
					[expr $gvar(dy) - ($i * $gvar(border))]\
					-fill $gvar(c_bg) -outline $gvar(c_bg_border)
	.c create arc	[expr $gvar(border) * $i]\
					[expr $gvar(border) * $i]\
					[expr $gvar(dx) - ($i * $gvar(border))]\
					[expr $gvar(dy) - ($i * $gvar(border))]\
					-fill $gvar(c_minute) -outline $gvar(c_minute)\
					-start 90 -tag minute
	grid 60 5 [expr $gvar(dx)/2 - ($i * $gvar(border))] [expr $gvar(dx)/2 - (($i+1) * $gvar(border))]
	# Stundenkreis
	incr i
	.c create oval	[expr $gvar(border) * $i]\
					[expr $gvar(border) * $i]\
					[expr $gvar(dx) - ($i * $gvar(border))]\
					[expr $gvar(dy) - ($i * $gvar(border))]\
					-fill $gvar(c_bg) -outline $gvar(c_bg_border)
	.c create arc	[expr $gvar(border) * $i]\
					[expr $gvar(border) * $i]\
					[expr $gvar(dx) - ($i * $gvar(border))]\
					[expr $gvar(dy) - ($i * $gvar(border))]\
					-fill $gvar(c_hour) -outline $gvar(c_hour)\
					-start 270 -tag hour
	grid 24 3 [expr $gvar(dx)/2 - ($i * $gvar(border))] [expr $gvar(dx)/2 - (($i+1) * $gvar(border))]
	# Kreis Sonnenaufgang/-untergang
	incr i
	.c create oval	[expr $gvar(border) * $i]\
					[expr $gvar(border) * $i]\
					[expr $gvar(dx) - ($i * $gvar(border))]\
					[expr $gvar(dy) - ($i * $gvar(border))]\
					-fill $gvar(c_bg) -outline $gvar(c_bg_border)
	.c create arc	[expr $gvar(border) * $i]\
					[expr $gvar(border) * $i]\
					[expr $gvar(dx) - ($i * $gvar(border))]\
					[expr $gvar(dy) - ($i * $gvar(border))]\
					-fill $gvar(c_sun) -outline $gvar(c_sun)\
					-start 0 -tag sun
	grid 24 3 [expr $gvar(dx)/2 - ($i * $gvar(border))] [expr $gvar(dx)/2 - (($i+1) * $gvar(border))]
	# letzter Innenkreis				
	incr i
	.c create oval	[expr $gvar(border) * $i]\
					[expr $gvar(border) * $i]\
					[expr $gvar(dx) - ($i * $gvar(border))]\
					[expr $gvar(dy) - ($i * $gvar(border))]\
					-fill $gvar(c_bg) -outline $gvar(c_bg_border)

# ******************************************************************
# *********************** main *************************************
# ******************************************************************
every 1000 {update_clock .c}
bind . <Escape> {exit}
